@@ -1,6 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
 const Pango = imports.gi.Pango;
 const GLib = imports.gi.GLib;
@@ -23,6 +22,7 @@ const Workspace = imports.ui.workspace;
 
 const APPICON_SIZE = 48;
 const WELL_MAX_COLUMNS = 8;
+const MENU_POPUP_TIMEOUT = 600;
 
 function AllAppView() {
     this._init();
@@ -286,17 +286,12 @@ BaseAppSearchProvider.prototype = {
 
     activateResult: function(id) {
         let app = this._appSys.get_app(id);
-        let windows = app.get_windows();
-
-        if (windows.length > 0)
-            Main.activateWindow(windows[0]);
-        else
-            app.launch();
+        app.activate();
     },
 
     dragActivateResult: function(id) {
         let app = this._appSys.get_app(id);
-        app.launch();
+        app.open_new_window();
     }
 };
 
@@ -328,7 +323,7 @@ AppSearchProvider.prototype = {
     },
 
     expandSearch: function(terms) {
-        log("TODO expand search");
+        log('TODO expand search');
     }
 };
 
@@ -391,7 +386,6 @@ function AppWellIcon(app) {
 AppWellIcon.prototype = {
     _init : function(app) {
         this.app = app;
-        this._running = false;
         this.actor = new St.Clickable({ style_class: 'app-well-app',
                                          reactive: true,
                                          x_fill: true,
@@ -404,68 +398,73 @@ AppWellIcon.prototype = {
         this.actor.connect('clicked', Lang.bind(this, this._onClicked));
         this._menu = null;
 
-        this._draggable = DND.makeDraggable(this.actor,
-                                            { manualMode: true });
-        this._dragStartX = null;
-        this._dragStartY = null;
+        this._draggable = DND.makeDraggable(this.actor);
+        this._draggable.connect('drag-begin', Lang.bind(this,
+            function () {
+                this._removeMenuTimeout();
+            }));
 
         this.actor.connect('button-press-event', Lang.bind(this, this._onButtonPress));
-        this.actor.connect('notify::hover', Lang.bind(this, this._onHoverChange));
         this.actor.connect('show', Lang.bind(this, this._onShow));
         this.actor.connect('hide', Lang.bind(this, this._onHideDestroy));
         this.actor.connect('destroy', Lang.bind(this, this._onHideDestroy));
 
-        this._appWindowChangedId = 0;
+        this._stateChangedId = 0;
+        this._menuTimeoutId = 0;
     },
 
     _onShow: function() {
-        this._appWindowChangedId = this.app.connect('windows-changed',
-                                                    Lang.bind(this,
-                                                              this._updateStyleClass));
-        this._updateStyleClass();
+        this._stateChangedId = this.app.connect('notify::state',
+                                                Lang.bind(this,
+                                                          this._onStateChanged));
+        this._onStateChanged();
     },
 
     _onHideDestroy: function() {
-        if (this._appWindowChangedId > 0)
-            this.app.disconnect(this._appWindowChangedId);
+        if (this._stateChangedId > 0)
+            this.app.disconnect(this._stateChangedId);
+        this._removeMenuTimeout();
     },
 
-    _updateStyleClass: function() {
-        let windows = this.app.get_windows();
-        let running = windows.length > 0;
-        this._running = running;
-        let style = "app-well-app";
-        if (this._running)
-            style += " running";
-        if (this._selected)
-            style += " selected";
-        this.actor.style_class = style;
+    _removeMenuTimeout: function() {
+        if (this._menuTimeoutId > 0) {
+            Mainloop.source_remove(this._menuTimeoutId);
+            this._menuTimeoutId = 0;
+        }
+    },
+
+    _onStateChanged: function() {
+        if (this.app.state != Shell.AppState.STOPPED)
+            this.actor.add_style_class_name('running');
+        else
+            this.actor.remove_style_class_name('running');
     },
 
     _onButtonPress: function(actor, event) {
-        let [stageX, stageY] = event.get_coords();
-        this._dragStartX = stageX;
-        this._dragStartY = stageY;
-    },
-
-    _onHoverChange: function(actor) {
-        let hover = this.actor.hover;
-        if (!hover) {
-            if (this.actor.pressed && this._dragStartX != null) {
-                this.actor.fake_release();
-                this._draggable.startDrag(this._dragStartX, this._dragStartY,
-                                          global.get_current_time());
-            } else {
-                this._dragStartX = null;
-                this._dragStartY = null;
-            }
+        let button = event.get_button();
+        if (button == 1) {
+            this._removeMenuTimeout();
+            this._menuTimeoutId = Mainloop.timeout_add(MENU_POPUP_TIMEOUT,
+                Lang.bind(this, function() {
+                    this.popupMenu(button);
+                }));
         }
     },
 
     _onClicked: function(actor, event) {
+        this._removeMenuTimeout();
+
         let button = event.get_button();
         if (button == 1) {
             this._onActivate(event);
+        } else if (button == 2) {
+            let newWorkspace = Main.overview.workspaces.addWorkspace();
+            if (newWorkspace != null) {
+                newWorkspace.activate(global.get_current_time());
+                this.emit('launching');
+                this.app.open_new_window();
+                Main.overview.hide();
+            }
         } else if (button == 3) {
             // Don't bind to the right click here; we want left click outside the
             // area to deactivate as well.
@@ -479,6 +478,9 @@ AppWellIcon.prototype = {
     },
 
     popupMenu: function(activatingButton) {
+        this._removeMenuTimeout();
+        this.actor.fake_release();
+
         if (!this._menu) {
             this._menu = new AppIconMenu(this);
             this._menu.connect('highlight-window', Lang.bind(this, function (menu, window) {
@@ -501,11 +503,6 @@ AppWellIcon.prototype = {
         return false;
     },
 
-    activateMostRecentWindow: function () {
-        let mostRecentWindow = this.app.get_windows()[0];
-        Main.activateWindow(mostRecentWindow);
-    },
-
     highlightWindow: function(metaWindow) {
         if (this._didActivateWindow)
             return;
@@ -518,13 +515,17 @@ AppWellIcon.prototype = {
         if (metaWindow) {
             this._didActivateWindow = true;
             Main.activateWindow(metaWindow);
-        } else
+        } else {
             Main.overview.hide();
+        }
     },
 
     setSelected: function (isSelected) {
         this._selected = isSelected;
-        this._updateStyleClass();
+        if (this._selected)
+            this.actor.add_style_class_name('selected');
+        else
+            this.actor.remove_style_class_name('selected');
     },
 
     _onMenuPoppedUp: function() {
@@ -536,6 +537,8 @@ AppWellIcon.prototype = {
     },
 
     _onMenuPoppedDown: function() {
+        this.actor.sync_hover();
+
         if (this._didActivateWindow)
             return;
         if (!this._setWindowSelection)
@@ -546,38 +549,24 @@ AppWellIcon.prototype = {
     },
 
     _getRunning: function() {
-        return this.app.get_windows().length > 0;
+        return this.app.state != Shell.AppState.STOPPED;
     },
 
     _onActivate: function (event) {
-        let running = this._getRunning();
         this.emit('launching');
+        let modifiers = Shell.get_event_state(event);
 
-        if (!running) {
-            this.app.launch();
-            Main.overview.hide();
+        if (modifiers & Clutter.ModifierType.CONTROL_MASK
+            && this.app.state == Shell.AppState.RUNNING) {
+            this.app.open_new_window();
         } else {
-            let modifiers = Shell.get_event_state(event);
-
-            if (modifiers & Clutter.ModifierType.CONTROL_MASK) {
-                this.app.launch();
-                Main.overview.hide();
-            } else {
-                this.activateMostRecentWindow();
-            }
+            this.app.activate();
         }
+        Main.overview.hide();
     },
 
     shellWorkspaceLaunch : function() {
-        // Here we just always launch the application again, even if we know
-        // it was already running.  For most applications this
-        // should have the effect of creating a new window, whether that's
-        // a second process (in the case of Calculator) or IPC to existing
-        // instance (Firefox).  There are a few less-sensical cases such
-        // as say Pidgin, but ideally what we do there is have the app
-        // express to us that it doesn't do relaunch=new-window in the
-        // .desktop file.
-        this.app.launch();
+        this.app.open_new_window();
     },
 
     getDragActor: function() {
@@ -600,26 +589,19 @@ AppIconMenu.prototype = {
     _init: function(source) {
         this._source = source;
 
-        this._arrowSize = 4; // CSS default
-        this._spacing = 0; // CSS default
-
-        this._dragStartX = 0;
-        this._dragStartY = 0;
-
         this.actor = new Shell.GenericContainer({ reactive: true });
         this.actor.connect('get-preferred-width', Lang.bind(this, this._getPreferredWidth));
         this.actor.connect('get-preferred-height', Lang.bind(this, this._getPreferredHeight));
         this.actor.connect('allocate', Lang.bind(this, this._allocate));
 
-        this._windowContainerBox = new St.Bin({ style_class: 'app-well-menu' });
-        this._windowContainer = new Shell.Menu({ orientation: Big.BoxOrientation.VERTICAL,
-                                                  width: Main.overview._dash.actor.width });
-        this._windowContainerBox.set_child(this._windowContainer);
+        this._windowContainer = new Shell.Menu({ style_class: 'app-well-menu',
+                                                 vertical: true,
+                                                 width: Main.overview._dash.actor.width });
         this._windowContainer.connect('unselected', Lang.bind(this, this._onItemUnselected));
         this._windowContainer.connect('selected', Lang.bind(this, this._onItemSelected));
         this._windowContainer.connect('cancelled', Lang.bind(this, this._onWindowSelectionCancelled));
         this._windowContainer.connect('activate', Lang.bind(this, this._onItemActivate));
-        this.actor.add_actor(this._windowContainerBox);
+        this.actor.add_actor(this._windowContainer);
 
         // Stay popped up on release over application icon
         this._windowContainer.set_persistent_source(this._source.actor);
@@ -629,16 +611,9 @@ AppIconMenu.prototype = {
         this._windowContainer.connect('leave-event', Lang.bind(this, this._onMenuLeave));
         this._windowContainer.connect('button-release-event', Lang.bind(this, this._onMenuButtonRelease));
 
-        this._borderColor = new Clutter.Color();
-        this._backgroundColor = new Clutter.Color();
-        this._windowContainerBox.connect('style-changed', Lang.bind(this, this._onStyleChanged));
-
-        this._arrow = new St.DrawingArea();
+        this._arrow = new St.DrawingArea({ style_class: 'app-well-menu-arrow' });
         this._arrow.connect('repaint', Lang.bind(this, function (area) {
-            Shell.draw_box_pointer(area,
-                                   Shell.PointerDirection.LEFT,
-                                   this._borderColor,
-                                   this._backgroundColor);
+            Shell.draw_box_pointer(area, Shell.PointerDirection.LEFT);
         }));
         this.actor.add_actor(this._arrow);
 
@@ -649,43 +624,44 @@ AppIconMenu.prototype = {
         }));
         source.actor.connect('destroy', Lang.bind(this, function () { this.actor.destroy(); }));
 
-        global.stage.add_actor(this.actor);
+        Main.uiGroup.add_actor(this.actor);
     },
 
     _getPreferredWidth: function(actor, forHeight, alloc) {
-        let [min, natural] = this._windowContainerBox.get_preferred_width(forHeight);
-        min += this._arrowSize;
-        natural += this._arrowSize;
-        alloc.min_size = min;
-        alloc.natural_size = natural;
+        let [menuMin, menuNatural] = this._windowContainer.get_preferred_width(forHeight);
+        let [arrowMin, arrowNatural] = this._arrow.get_preferred_width(forHeight);
+        alloc.min_size = menuMin + arrowMin;
+        alloc.natural_size = menuNatural + arrowNatural;
     },
 
     _getPreferredHeight: function(actor, forWidth, alloc) {
-        let [min, natural] = this._windowContainerBox.get_preferred_height(forWidth);
+        let [min, natural] = this._windowContainer.get_preferred_height(forWidth);
         alloc.min_size = min;
         alloc.natural_size = natural;
     },
 
     _allocate: function(actor, box, flags) {
         let childBox = new Clutter.ActorBox();
-        let themeNode = this._windowContainerBox.get_theme_node();
+        let themeNode = this._windowContainer.get_theme_node();
 
         let width = box.x2 - box.x1;
         let height = box.y2 - box.y1;
 
+        let [arrowMinWidth, arrowWidth] = this._arrow.get_preferred_width(height);
+
         childBox.x1 = 0;
-        childBox.x2 = this._arrowSize;
-        childBox.y1 = Math.floor((height / 2) - (this._arrowSize / 2));
-        childBox.y2 = childBox.y1 + this._arrowSize;
+        childBox.x2 = arrowWidth;
+        childBox.y1 = Math.floor((height / 2) - (arrowWidth / 2));
+        childBox.y2 = childBox.y1 + arrowWidth;
         this._arrow.allocate(childBox, flags);
 
         // Ensure the arrow is above the border area
         let border = themeNode.get_border_width(St.Side.LEFT);
-        childBox.x1 = this._arrowSize - border;
+        childBox.x1 = arrowWidth - border;
         childBox.x2 = width;
         childBox.y1 = 0;
         childBox.y2 = height;
-        this._windowContainerBox.allocate(childBox, flags);
+        this._windowContainer.allocate(childBox, flags);
     },
 
     _redisplay: function() {
@@ -737,8 +713,8 @@ AppIconMenu.prototype = {
     },
 
     _appendSeparator: function () {
-        let bin = new St.Bin({ style_class: "app-well-menu-separator" });
-        this._windowContainer.append_separator(bin, Big.BoxPackFlags.NONE);
+        let bin = new St.Bin({ style_class: 'app-well-menu-separator' });
+        this._windowContainer.add_actor(bin);
     },
 
     _appendMenuItem: function(labelText) {
@@ -746,7 +722,7 @@ AppIconMenu.prototype = {
                                       reactive: true });
         let label = new St.Label({ text: labelText });
         box.add(label);
-        this._windowContainer.append(box, Big.BoxPackFlags.NONE);
+        this._windowContainer.add_actor(box);
         return box;
     },
 
@@ -797,12 +773,12 @@ AppIconMenu.prototype = {
 
     _updateHighlight: function (item) {
         if (this._highlightedItem) {
-            this._highlightedItem.set_style_pseudo_class(null);
+            this._highlightedItem.remove_style_pseudo_class('hover');
             this.emit('highlight-window', null);
         }
         this._highlightedItem = item;
         if (this._highlightedItem) {
-            item.set_style_pseudo_class('hover');
+            item.add_style_pseudo_class('hover');
             let window = this._highlightedItem._window;
             if (window)
                 this.emit('highlight-window', window);
@@ -848,7 +824,7 @@ AppIconMenu.prototype = {
             let metaWindow = child._window;
             this.emit('activate-window', metaWindow);
         } else if (child == this._newWindowMenuItem) {
-            this._source.app.launch();
+            this._source.app.open_new_window();
             this.emit('activate-window', null);
         } else if (child == this._toggleFavoriteMenuItem) {
             let favs = AppFavorites.getAppFavorites();
@@ -864,28 +840,6 @@ AppIconMenu.prototype = {
     _onWindowSelectionCancelled: function () {
         this.emit('highlight-window', null);
         this.popdown();
-    },
-
-    _onStyleChanged: function() {
-        let themeNode = this._windowContainerBox.get_theme_node();
-        let [success, len] = themeNode.get_length('-shell-arrow-size', false);
-        if (success) {
-            this._arrowSize = len;
-            this.actor.queue_relayout();
-        }
-        [success, len] = themeNode.get_length('-shell-menu-spacing', false)
-        if (success) {
-            this._windowContainer.spacing = len;
-        }
-        let color = new Clutter.Color();
-        if (themeNode.get_background_color(color)) {
-            this._backgroundColor = color;
-            color = new Clutter.Color();
-        }
-        if (themeNode.get_border_color(St.Side.LEFT, color)) {
-            this._borderColor = color;
-        }
-        this._arrow.queue_repaint();
     }
 };
 Signals.addSignalMethods(AppIconMenu.prototype);
@@ -896,7 +850,7 @@ function WellGrid() {
 
 WellGrid.prototype = {
     _init: function() {
-        this.actor = new St.BoxLayout({ name: "dashAppWell", vertical: true });
+        this.actor = new St.BoxLayout({ name: 'dashAppWell', vertical: true });
         // Pulled from CSS, but hardcode some defaults here
         this._spacing = 0;
         this._item_size = 48;
@@ -1027,15 +981,12 @@ AppWell.prototype = {
 
         this._favorites = [];
 
-        this.actor = new Big.Box({ orientation: Big.BoxOrientation.VERTICAL,
-                                   x_align: Big.BoxAlignment.CENTER });
+        this._grid = new WellGrid();
+        this.actor = this._grid.actor;
         this.actor._delegate = this;
 
         this._workId = Main.initializeDeferredWork(this.actor, Lang.bind(this, this._redisplay));
 
-        this._grid = new WellGrid();
-        this.actor.append(this._grid.actor, Big.BoxPackFlags.EXPAND);
-
         this._tracker = Shell.WindowTracker.get_default();
         this._appSystem = Shell.AppSystem.get_default();
 
@@ -1061,7 +1012,7 @@ AppWell.prototype = {
         let favorites = AppFavorites.getAppFavorites().getFavoriteMap();
 
         /* hardcode here pending some design about how exactly desktop contexts behave */
-        let contextId = "";
+        let contextId = '';
 
         let running = this._tracker.get_running_apps(contextId);
         let runningIds = this._appIdListToHash(running);
