@@ -1,6 +1,5 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
-const Big = imports.gi.Big;
 const Clutter = imports.gi.Clutter;
 const Gtk = imports.gi.Gtk;
 const Mainloop = imports.mainloop;
@@ -23,35 +22,8 @@ const Search = imports.ui.search;
 // 25 search results (per result type) should be enough for everyone
 const MAX_RENDERED_SEARCH_RESULTS = 25;
 
-const DEFAULT_PADDING = 4;
-const DEFAULT_SPACING = 4;
-
-const BACKGROUND_COLOR = new Clutter.Color();
-BACKGROUND_COLOR.from_pixel(0x000000c0);
-
-const PRELIGHT_COLOR = new Clutter.Color();
-PRELIGHT_COLOR.from_pixel(0x4f6fadaa);
-
-const TEXT_COLOR = new Clutter.Color();
-TEXT_COLOR.from_pixel(0x5f5f5fff);
-const BRIGHTER_TEXT_COLOR = new Clutter.Color();
-BRIGHTER_TEXT_COLOR.from_pixel(0xbbbbbbff);
-const BRIGHT_TEXT_COLOR = new Clutter.Color();
-BRIGHT_TEXT_COLOR.from_pixel(0xffffffff);
-const SEARCH_TEXT_COLOR = new Clutter.Color();
-SEARCH_TEXT_COLOR.from_pixel(0x333333ff);
-
-const SEARCH_CURSOR_COLOR = BRIGHT_TEXT_COLOR;
-const HIGHLIGHTED_SEARCH_CURSOR_COLOR = SEARCH_TEXT_COLOR;
-
-const SEARCH_BORDER_BOTTOM_COLOR = new Clutter.Color();
-SEARCH_BORDER_BOTTOM_COLOR.from_pixel(0x191919ff);
-
-const BROWSE_ACTIVATED_BG = new Clutter.Color();
-BROWSE_ACTIVATED_BG.from_pixel(0x303030f0);
-
-const DOCS = "docs";
-const PLACES = "places";
+const DOCS = 'docs';
+const PLACES = 'places';
 
 /*
  * Returns the index in an array of a given length that is obtained
@@ -82,7 +54,7 @@ Pane.prototype = {
     _init: function () {
         this._open = false;
 
-        this.actor = new St.BoxLayout({ style_class: "dash-pane",
+        this.actor = new St.BoxLayout({ style_class: 'dash-pane',
                                          vertical: true,
                                          reactive: true });
         this.actor.connect('button-press-event', Lang.bind(this, function (a, e) {
@@ -92,13 +64,8 @@ Pane.prototype = {
 
         let chromeTop = new St.BoxLayout();
 
-        let closeIcon = new St.Button({ style_class: "dash-pane-close" });
-        closeIcon.connect('clicked', Lang.bind(this, function (b, e) {
-            this.close();
-        }));
         let dummy = new St.Bin();
         chromeTop.add(dummy, { expand: true });
-        chromeTop.add(closeIcon, { x_align: St.Align.END });
         this.actor.add(chromeTop);
 
         this.content = new St.BoxLayout({ vertical: true });
@@ -137,7 +104,7 @@ Pane.prototype = {
         else
             this.open();
     }
-}
+};
 Signals.addSignalMethods(Pane.prototype);
 
 function ResultArea(displayType, flags) {
@@ -146,17 +113,15 @@ function ResultArea(displayType, flags) {
 
 ResultArea.prototype = {
     _init : function(displayType, flags) {
-        this.actor = new Big.Box({ orientation: Big.BoxOrientation.VERTICAL });
-        this.resultsContainer = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL,
-                                              spacing: DEFAULT_PADDING
-                                            });
-        this.actor.append(this.resultsContainer, Big.BoxPackFlags.EXPAND);
+        this.actor = new St.BoxLayout({ vertical: true });
+        this.resultsContainer = new St.BoxLayout({ style_class: 'dash-results-container' });
+        this.actor.add(this.resultsContainer, { expand: true });
 
         this.display = _createDisplay(displayType, flags);
-        this.resultsContainer.append(this.display.actor, Big.BoxPackFlags.EXPAND);
+        this.resultsContainer.add(this.display.actor, { expand: true });
         this.display.load();
     }
-}
+};
 
 // Utility function shared between ResultPane and the DocDisplay in the main dash.
 // Connects to the detail signal of the display, and on-demand creates a new
@@ -172,7 +137,7 @@ function createPaneForDetails(dash, display) {
                     detailPane.destroyContent();
                 }
             }));
-            dash._addPane(detailPane);
+            dash._addPane(detailPane, St.Align.START);
         }
 
         if (index >= 0) {
@@ -212,7 +177,7 @@ ResultPane.prototype = {
         }));
         return resultArea.display;
     }
-}
+};
 
 function SearchEntry() {
     this._init();
@@ -220,76 +185,44 @@ function SearchEntry() {
 
 SearchEntry.prototype = {
     _init : function() {
-        this.actor = new St.BoxLayout({ name: "searchEntry",
-                                        reactive: true });
-        let box = new Big.Box({ orientation: Big.BoxOrientation.HORIZONTAL,
-                                y_align: Big.BoxAlignment.CENTER });
-        this.actor.add(box, { expand: true });
-        this.actor.connect('button-press-event', Lang.bind(this, function () {
-            this._resetTextState(true);
-            return false;
-        }));
+        this.actor = new St.Entry({ name: 'searchEntry',
+                                    hint_text: _("Find") });
+        this.entry = this.actor.clutter_text;
+
+        this.actor.clutter_text.connect('text-changed', Lang.bind(this,
+            function() {
+                if (this.isActive())
+                    this.actor.set_secondary_icon_from_file(global.imagedir +
+                                                            'close-black.svg');
+                else
+                    this.actor.set_secondary_icon_from_file(null);
+            }));
+        this.actor.connect('secondary-icon-clicked', Lang.bind(this,
+            function() {
+                this.reset();
+            }));
+        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
 
         this.pane = null;
 
-        this._defaultText = _("Find...");
+        this._capturedEventId = 0;
+    },
 
-        let textProperties = { font_name: "Sans 16px" };
-        let entryProperties = { editable: true,
-                                activatable: true,
-                                single_line_mode: true,
-                                color: SEARCH_TEXT_COLOR,
-                                cursor_color: SEARCH_CURSOR_COLOR };
-        Lang.copyProperties(textProperties, entryProperties);
-        this.entry = new Clutter.Text(entryProperties);
+    show: function() {
+        if (this._capturedEventId == 0)
+            this._capturedEventId = global.stage.connect('captured-event',
+                                 Lang.bind(this, this._onCapturedEvent));
+        this.entry.set_cursor_visible(true);
+        this.entry.set_selection(0, 0);
+    },
 
-        this.entry.connect('notify::text', Lang.bind(this, function () {
-            this._resetTextState(false);
-        }));
-        box.append(this.entry, Big.BoxPackFlags.EXPAND);
-
-        // Mark as editable just to get a cursor
-        let defaultTextProperties = { ellipsize: Pango.EllipsizeMode.END,
-                                      text: this._defaultText,
-                                      editable: true,
-                                      color: TEXT_COLOR,
-                                      cursor_visible: false,
-                                      single_line_mode: true };
-        Lang.copyProperties(textProperties, defaultTextProperties);
-        this._defaultText = new Clutter.Text(defaultTextProperties);
-        box.add_actor(this._defaultText);
-        this.entry.connect('notify::allocation', Lang.bind(this, function () {
-            this._repositionDefaultText();
-        }));
-
-        this._iconBox = new Big.Box({ x_align: Big.BoxAlignment.CENTER,
-                                      y_align: Big.BoxAlignment.CENTER,
-                                      padding_right: 4 });
-        box.append(this._iconBox, Big.BoxPackFlags.END);
-
-        let magnifierUri = "file://" + global.imagedir + "magnifier.svg";
-        this._magnifierIcon = Shell.TextureCache.get_default().load_uri_sync(Shell.TextureCachePolicy.FOREVER,
-                                                                             magnifierUri, 18, 18);
-        let closeUri = "file://" + global.imagedir + "close-black.svg";
-        this._closeIcon = Shell.TextureCache.get_default().load_uri_sync(Shell.TextureCachePolicy.FOREVER,
-                                                                         closeUri, 18, 18);
-        this._closeIcon.reactive = true;
-        this._closeIcon.connect('button-press-event', Lang.bind(this, function () {
-            // Resetting this.entry.text will trigger notify::text signal which will
-            // result in this._resetTextState() being called, but we should not rely
-            // on that not short-circuiting if the text was already empty, so we call
-            // this._resetTextState() explicitly in that case.
-            if (this.entry.text == '')
-                this._resetTextState(false);
-            else
-                this.entry.text = '';
-
-            // Return true to stop the signal emission, so that this.actor doesn't get
-            // the button-press-event and re-highlight itself.
-            return true;
-        }));
-        this._repositionDefaultText();
-        this._resetTextState();
+    hide: function() {
+        if (this.isActive())
+            this.reset();
+        if (this._capturedEventId > 0) {
+            global.stage.disconnect(this._capturedEventId);
+            this._capturedEventId = 0;
+        }
     },
 
     setPane: function (pane) {
@@ -298,35 +231,93 @@ SearchEntry.prototype = {
 
     reset: function () {
         this.entry.text = '';
+        global.stage.set_key_focus(null);
+        this.entry.set_cursor_visible(true);
+        this.entry.set_selection(0, 0);
     },
 
     getText: function () {
-        return this.entry.text;
+        return this.entry.get_text().replace(/^\s+/g, '').replace(/\s+$/g, '');
     },
 
-    _resetTextState: function (searchEntryClicked) {
-        let text = this.getText();
-        this._iconBox.remove_all();
-        // We highlight the search box if the user starts typing in it
-        // or just clicks in it to indicate that the search is active.
-        if (text != '' || searchEntryClicked) {
-            if (!searchEntryClicked)
-                this._defaultText.hide();
-            this._iconBox.append(this._closeIcon, Big.BoxPackFlags.NONE);
-            this.actor.set_style_pseudo_class('active');
-            this.entry.cursor_color = HIGHLIGHTED_SEARCH_CURSOR_COLOR;
-        } else {
-            this._defaultText.show();
-            this._iconBox.append(this._magnifierIcon, Big.BoxPackFlags.NONE);
-            this.actor.set_style_pseudo_class(null);
-            this.entry.cursor_color = SEARCH_CURSOR_COLOR;
+    // some search term has been entered
+    isActive: function() {
+        return this.actor.get_text() != '';
+    },
+
+    // the entry does not show the hint
+    _isActivated: function() {
+        return this.entry.text == this.actor.get_text();
+    },
+
+    _onCapturedEvent: function(actor, event) {
+        let source = event.get_source();
+        let panelEvent = false;
+
+        if (source) {
+            let parent = source;
+            do {
+                if (parent == Main.panel.actor)
+                    break;
+            } while ((parent = parent.get_parent()) != null);
+            panelEvent = (parent != null);
+        }
+
+        switch (event.type()) {
+            case Clutter.EventType.BUTTON_PRESS:
+                // the user clicked outside after activating the entry, but
+                // with no search term entered - cancel the search
+                if (source != this.entry && this.entry.text == '') {
+                    this.reset();
+                    // allow only panel events to continue
+                    return !panelEvent;
+                }
+                return false;
+            case Clutter.EventType.KEY_PRESS:
+                // If neither the stage nor our entry have key focus, some
+                // "special" actor grabbed the focus (run dialog, looking
+                // glass); we don't want to interfere with that
+                let focus = global.stage.get_key_focus();
+                if (focus != global.stage && focus != this.entry)
+                    return false;
+
+                let sym = event.get_key_symbol();
+
+                // If we have an active search, Escape cancels it - if we
+                // haven't, the key is ignored
+                if (sym == Clutter.Escape)
+                    if (this._isActivated()) {
+                        this.reset();
+                        return true;
+                    } else {
+                        return false;
+                    }
+
+                 // Ignore non-printable keys
+                 if (!Clutter.keysym_to_unicode(sym))
+                     return false;
+
+                // Search started - move the key focus to the entry and
+                // "repeat" the event
+                if (!this._isActivated()) {
+                    global.stage.set_key_focus(this.entry);
+                    this.entry.event(event, false);
+                }
+
+                return false;
+            default:
+                // Suppress all other events outside the panel while the entry
+                // is activated and no search has been entered - any click
+                // outside the entry will cancel the search
+                return (this.entry.text == '' && !panelEvent);
         }
     },
 
-    _repositionDefaultText: function () {
-        // Offset a little to show the cursor
-        this._defaultText.set_position(this.entry.x + 4, this.entry.y);
-        this._defaultText.set_size(this.entry.width, this.entry.height);
+    _onDestroy: function() {
+        if (this._capturedEventId > 0) {
+            global.stage.disconnect(this._capturedEventId);
+            this._capturedEventId = 0;
+        }
     }
 };
 Signals.addSignalMethods(SearchEntry.prototype);
@@ -363,7 +354,10 @@ SearchResult.prototype = {
     },
 
     setSelected: function(selected) {
-        this._content.set_style_pseudo_class(selected ? 'selected' : null);
+        if (selected)
+            this._content.add_style_pseudo_class('selected');
+        else
+            this._content.remove_style_pseudo_class('selected');
     },
 
     activate: function() {
@@ -389,7 +383,7 @@ SearchResult.prototype = {
         else
             this.provider.activateResult(this.metaInfo.id);
     }
-}
+};
 
 function OverflowSearchResults(provider) {
     this._init(provider);
@@ -403,6 +397,10 @@ OverflowSearchResults.prototype = {
         this.actor = new St.OverflowBox({ style_class: 'dash-search-section-list-results' });
     },
 
+    getVisibleResultCount: function() {
+        return this.actor.get_n_visible();
+    },
+
     renderResults: function(results, terms) {
         for (let i = 0; i < results.length && i < MAX_RENDERED_SEARCH_RESULTS; i++) {
             let result = results[i];
@@ -412,10 +410,6 @@ OverflowSearchResults.prototype = {
         }
     },
 
-    getVisibleCount: function() {
-        return this.actor.get_n_visible();
-    },
-
     selectIndex: function(index) {
         let nVisible = this.actor.get_n_visible();
         let children = this.actor.get_children();
@@ -432,8 +426,14 @@ OverflowSearchResults.prototype = {
         targetActor._delegate.setSelected(true);
         this.selectionIndex = index;
         return true;
+    },
+
+    activateSelected: function() {
+        let children = this.actor.get_children();
+        let targetActor = children[this.selectionIndex];
+        targetActor._delegate.activate();
     }
-}
+};
 
 function SearchResults(searchSystem) {
     this._init(searchSystem);
@@ -530,7 +530,7 @@ SearchResults.prototype = {
             let meta = this._metaForProvider(provider);
             meta.actor.show();
             meta.resultDisplay.renderResults(providerResults, terms);
-            meta.count.set_text(""+providerResults.length);
+            meta.count.set_text('' + providerResults.length);
         }
 
         this.selectDown(false);
@@ -546,7 +546,7 @@ SearchResults.prototype = {
         let success;
         let index = resultDisplay.getSelectionIndex();
         if (up && index == -1)
-            index = resultDisplay.getVisibleCount() - 1;
+            index = resultDisplay.getVisibleResultCount() - 1;
         else if (up)
             index = index - 1;
         else
@@ -597,11 +597,10 @@ SearchResults.prototype = {
             return;
         let meta = this._providerMeta[current];
         let resultDisplay = meta.resultDisplay;
-        let children = resultDisplay.actor.get_children();
-        let targetActor = children[resultDisplay.getSelectionIndex()];
-        targetActor._delegate.activate();
+        resultDisplay.activateSelected();
+        Main.overview.hide();
     }
-}
+};
 
 function MoreLink() {
     this._init();
@@ -609,11 +608,11 @@ function MoreLink() {
 
 MoreLink.prototype = {
     _init : function () {
-        this.actor = new St.BoxLayout({ style_class: "more-link",
+        this.actor = new St.BoxLayout({ style_class: 'more-link',
                                         reactive: true });
         this.pane = null;
 
-        this._expander = new St.Bin({ style_class: "more-link-expander" });
+        this._expander = new St.Bin({ style_class: 'more-link-expander' });
         this.actor.add(this._expander, { expand: true, y_fill: false });
     },
 
@@ -629,13 +628,13 @@ MoreLink.prototype = {
     setPane: function (pane) {
         this._pane = pane;
         this._pane.connect('open-state-changed', Lang.bind(this, function(pane, isOpen) {
-            if (!isOpen)
-                this._expander.style_class = 'more-link-expander';
+            if (isOpen)
+                this._expander.add_style_class_name('open');
             else
-                this._expander.style_class = 'more-link-expander open';
+                this._expander.remove_style_class_name('open');
         }));
     }
-}
+};
 
 Signals.addSignalMethods(MoreLink.prototype);
 
@@ -645,11 +644,11 @@ function BackLink() {
 
 BackLink.prototype = {
     _init : function () {
-        this.actor = new St.Button({ style_class: "section-header-back",
+        this.actor = new St.Button({ style_class: 'section-header-back',
                                       reactive: true });
-        this.actor.set_child(new St.Bin({ style_class: "section-header-back-image" }));
+        this.actor.set_child(new St.Bin({ style_class: 'section-header-back-image' }));
     }
-}
+};
 
 function SectionHeader(title, suppressBrowse) {
     this._init(title, suppressBrowse);
@@ -657,12 +656,12 @@ function SectionHeader(title, suppressBrowse) {
 
 SectionHeader.prototype = {
     _init : function (title, suppressBrowse) {
-        this.actor = new St.Bin({ style_class: "section-header",
+        this.actor = new St.Bin({ style_class: 'section-header',
                                   x_align: St.Align.START,
                                   x_fill: true,
                                   y_fill: true,
                                   reactive: !suppressBrowse });
-        this._innerBox = new St.BoxLayout({ style_class: "section-header-inner" });
+        this._innerBox = new St.BoxLayout({ style_class: 'section-header-inner' });
         this.actor.set_child(this._innerBox);
 
         this.backLink = new BackLink();
@@ -672,12 +671,12 @@ SectionHeader.prototype = {
             this.emit('back-link-activated');
         }));
 
-        let textBox = new St.BoxLayout({ style_class: "section-text-content" });
-        this.text = new St.Label({ style_class: "section-title",
+        let textBox = new St.BoxLayout({ style_class: 'section-text-content' });
+        this.text = new St.Label({ style_class: 'section-title',
                                    text: title });
         textBox.add(this.text, { x_align: St.Align.START });
 
-        this.countText = new St.Label({ style_class: "section-count" });
+        this.countText = new St.Label({ style_class: 'section-count' });
         textBox.add(this.countText, { expand: true, x_fill: false, x_align: St.Align.END });
         this.countText.hide();
 
@@ -713,14 +712,14 @@ SectionHeader.prototype = {
     },
 
     setCountText : function(countText) {
-        if (countText == "") {
+        if (countText == '') {
             this.countText.hide();
         } else {
             this.countText.show();
             this.countText.text = countText;
         }
     }
-}
+};
 
 Signals.addSignalMethods(SectionHeader.prototype);
 
@@ -730,21 +729,21 @@ function SearchSectionHeader(title, onClick) {
 
 SearchSectionHeader.prototype = {
     _init : function(title, onClick) {
-        this.actor = new St.Button({ style_class: "dash-search-section-header",
+        this.actor = new St.Button({ style_class: 'dash-search-section-header',
                                       x_fill: true,
                                       y_fill: true });
         let box = new St.BoxLayout();
         this.actor.set_child(box);
-        let titleText = new St.Label({ style_class: "dash-search-section-title",
+        let titleText = new St.Label({ style_class: 'dash-search-section-title',
                                         text: title });
-        this.countText = new St.Label({ style_class: "dash-search-section-count" });
+        this.countText = new St.Label({ style_class: 'dash-search-section-count' });
 
         box.add(titleText);
         box.add(this.countText, { expand: true, x_fill: false, x_align: St.Align.END });
 
         this.actor.connect('clicked', onClick);
     }
-}
+};
 
 function Section(titleString, suppressBrowse) {
     this._init(titleString, suppressBrowse);
@@ -760,7 +759,7 @@ Section.prototype = {
                                            vertical: true });
         this.actor.add(this.content);
     }
-}
+};
 
 function Dash() {
     this._init();
@@ -777,14 +776,14 @@ Dash.prototype = {
         // of the Group actor ends up including the width of its hidden children, so we were getting a reactive object as
         // wide as the details pane that was blocking the clicks to the workspaces underneath it even when the details pane
         // was actually hidden.
-        this.actor = new St.BoxLayout({ name: "dash",
+        this.actor = new St.BoxLayout({ name: 'dash',
                                         vertical: true,
                                         reactive: true });
 
         // The searchArea just holds the entry
-        this.searchArea = new St.BoxLayout({ name: "dashSearchArea",
+        this.searchArea = new St.BoxLayout({ name: 'dashSearchArea',
                                              vertical: true });
-        this.sectionArea = new St.BoxLayout({ name: "dashSections",
+        this.sectionArea = new St.BoxLayout({ name: 'dashSections',
                                                vertical: true });
 
         this.actor.add(this.searchArea);
@@ -810,12 +809,11 @@ Dash.prototype = {
         this.actor.add(this.searchResults.actor);
         this.searchResults.actor.hide();
 
+        this._keyPressId = 0;
         this._searchTimeoutId = 0;
         this._searchEntry.entry.connect('text-changed', Lang.bind(this, function (se, prop) {
-            let text = this._searchEntry.getText();
-            text = text.replace(/^\s+/g, "").replace(/\s+$/g, "");
             let searchPreviouslyActive = this._searchActive;
-            this._searchActive = text != '';
+            this._searchActive = this._searchEntry.isActive();
             this._searchPending = this._searchActive && !searchPreviouslyActive;
             if (this._searchPending) {
                 this.searchResults.startingSearch();
@@ -846,35 +844,6 @@ Dash.prototype = {
             this.searchResults.activateSelected();
             return true;
         }));
-        this._searchEntry.entry.connect('key-press-event', Lang.bind(this, function (se, e) {
-            let symbol = e.get_key_symbol();
-            if (symbol == Clutter.Escape) {
-                // Escape will keep clearing things back to the desktop.
-                // If we have an active search, we remove it.
-                if (this._searchActive)
-                    this._searchEntry.reset();
-                // Next, if we're in one of the "more" modes or showing the details pane, close them
-                else if (this._activePane != null)
-                    this._activePane.close();
-                // Finally, just close the Overview entirely
-                else
-                    Main.overview.hide();
-                return true;
-            } else if (symbol == Clutter.Up) {
-                if (!this._searchActive)
-                    return true;
-                this.searchResults.selectUp(false);
-
-                return true;
-            } else if (symbol == Clutter.Down) {
-                if (!this._searchActive)
-                    return true;
-
-                this.searchResults.selectDown(false);
-                return true;
-            }
-            return false;
-        }));
 
         /***** Applications *****/
 
@@ -886,7 +855,7 @@ Dash.prototype = {
         this._appsSection.header.moreLink.connect('activated', Lang.bind(this, function (link) {
             if (this._allApps == null) {
                 this._allApps = new AppDisplay.AllAppDisplay();
-                this._addPane(this._allApps);
+                this._addPane(this._allApps, St.Align.START);
                 link.setPane(this._allApps);
            }
         }));
@@ -914,7 +883,7 @@ Dash.prototype = {
             if (this._moreDocsPane == null) {
                 this._moreDocsPane = new ResultPane(this);
                 this._moreDocsPane.packResults(DOCS);
-                this._addPane(this._moreDocsPane);
+                this._addPane(this._moreDocsPane, St.Align.END);
                 link.setPane(this._moreDocsPane);
            }
         }));
@@ -928,6 +897,40 @@ Dash.prototype = {
         this.sectionArea.add(this._docsSection.actor, { expand: true });
     },
 
+    _onKeyPress: function(stage, event) {
+        // If neither the stage nor the search entry have key focus, some
+        // "special" actor grabbed the focus (run dialog, looking glass);
+        // we don't want to interfere with that
+        let focus = stage.get_key_focus();
+        if (focus != stage && focus != this._searchEntry.entry)
+            return false;
+
+        let symbol = event.get_key_symbol();
+        if (symbol == Clutter.Escape) {
+            // If we're in one of the "more" modes or showing the
+            // details pane, close them
+            if (this._activePane != null)
+                this._activePane.close();
+            // Otherwise, just close the Overview entirely
+            else
+                Main.overview.hide();
+            return true;
+        } else if (symbol == Clutter.Up) {
+            if (!this._searchActive)
+                return true;
+            this.searchResults.selectUp(false);
+
+            return true;
+        } else if (symbol == Clutter.Down) {
+            if (!this._searchActive)
+                return true;
+
+            this.searchResults.selectDown(false);
+            return true;
+        }
+        return false;
+    },
+
     _doSearch: function () {
         this._searchTimeoutId = 0;
         let text = this._searchEntry.getText();
@@ -937,14 +940,21 @@ Dash.prototype = {
     },
 
     show: function() {
-        global.stage.set_key_focus(this._searchEntry.entry);
+        this._searchEntry.show();
+        if (this._keyPressId == 0)
+            this._keyPressId = global.stage.connect('key-press-event',
+                                                    Lang.bind(this, this._onKeyPress));
     },
 
     hide: function() {
         this._firstSelectAfterOverlayShow = true;
-        this._searchEntry.reset();
+        this._searchEntry.hide();
         if (this._activePane != null)
             this._activePane.close();
+        if (this._keyPressId > 0) {
+            global.stage.disconnect(this._keyPressId);
+            this._keyPressId = 0;
+        }
     },
 
     closePanes: function () {
@@ -952,7 +962,7 @@ Dash.prototype = {
             this._activePane.close();
     },
 
-    _addPane: function(pane) {
+    _addPane: function(pane, align) {
         pane.connect('open-state-changed', Lang.bind(this, function (pane, isOpen) {
             if (isOpen) {
                 if (pane != this._activePane && this._activePane != null) {
@@ -963,7 +973,7 @@ Dash.prototype = {
                 this._activePane = null;
             }
         }));
-        Main.overview.addPane(pane);
+        Main.overview.addPane(pane, align);
     }
 };
 Signals.addSignalMethods(Dash.prototype);
