@@ -2,9 +2,13 @@
 
 #include "config.h"
 
+#include "st.h"
 #include "shell-app-private.h"
 #include "shell-global.h"
+#include "shell-enum-types.h"
+#include "display.h"
 #include "st.h"
+#include "shell-window-tracker.h"
 
 #include <string.h>
 
@@ -21,15 +25,23 @@ struct _ShellApp
 
   ShellAppInfo *info;
 
+  guint32 last_user_time;
+
   guint workspace_switch_id;
 
-  gboolean window_sort_stale;
   GSList *windows;
 
+  ShellAppState state;
+  gboolean window_sort_stale : 1;
 };
 
 G_DEFINE_TYPE (ShellApp, shell_app, G_TYPE_OBJECT);
 
+enum {
+  PROP_0,
+  PROP_STATE
+};
+
 enum {
   WINDOWS_CHANGED,
   LAST_SIGNAL
@@ -37,6 +49,25 @@ enum {
 
 static guint shell_app_signals[LAST_SIGNAL] = { 0 };
 
+static void
+shell_app_get_property (GObject    *gobject,
+                        guint       prop_id,
+                        GValue     *value,
+                        GParamSpec *pspec)
+{
+  ShellApp *app = SHELL_APP (gobject);
+
+  switch (prop_id)
+    {
+    case PROP_STATE:
+      g_value_set_enum (value, app->state);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
+      break;
+    }
+}
+
 const char *
 shell_app_get_id (ShellApp *app)
 {
@@ -219,11 +250,201 @@ shell_app_is_transient (ShellApp *app)
   return shell_app_info_is_transient (app->info);
 }
 
-gboolean
-shell_app_launch (ShellApp  *self,
-                  GError   **error)
+typedef struct {
+  MetaWorkspace *workspace;
+  GSList **transients;
+} CollectTransientsData;
+
+static gboolean
+collect_transients_on_workspace (MetaWindow *window,
+                                 gpointer    datap)
 {
-  return shell_app_info_launch (self->info, error);
+  CollectTransientsData *data = datap;
+
+  if (data->workspace && meta_window_get_workspace (window) != data->workspace)
+    return TRUE;
+
+  *data->transients = g_slist_prepend (*data->transients, window);
+  return TRUE;
+}
+
+/* The basic idea here is that when we're targeting a window,
+ * if it has transients we want to pick the most recent one
+ * the user interacted with.
+ * This function makes raising GEdit with the file chooser
+ * open work correctly.
+ */
+static MetaWindow *
+find_most_recent_transient_on_same_workspace (MetaDisplay *display,
+                                              MetaWindow  *reference)
+{
+  GSList *transients, *transients_sorted, *iter;
+  MetaWindow *result;
+  CollectTransientsData data;
+
+  transients = NULL;
+  data.workspace = meta_window_get_workspace (reference);
+  data.transients = &transients;
+
+  meta_window_foreach_transient (reference, collect_transients_on_workspace, &data);
+
+  transients_sorted = meta_display_sort_windows_by_stacking (display, transients);
+  /* Reverse this so we're top-to-bottom (yes, we should probably change the order
+   * returned from the sort_windows_by_stacking function)
+   */
+  transients_sorted = g_slist_reverse (transients_sorted);
+  g_slist_free (transients);
+  transients = NULL;
+
+  result = NULL;
+  for (iter = transients_sorted; iter; iter = iter->next)
+    {
+      MetaWindow *window = iter->data;
+      MetaWindowType wintype = meta_window_get_window_type (window);
+
+      /* Don't want to focus UTILITY types, like the Gimp toolbars */
+      if (wintype == META_WINDOW_NORMAL ||
+          wintype == META_WINDOW_DIALOG)
+        {
+          result = window;
+          break;
+        }
+    }
+  g_slist_free (transients_sorted);
+  return result;
+}
+
+/**
+ * shell_app_activate_window:
+ * @app: a #ShellApp
+ * @window: (allow-none): Window to be focused
+ * @timestamp: Event timestamp
+ *
+ * Bring all windows for the given app to the foreground,
+ * but ensure that @window is on top.  If @window is %NULL,
+ * the window with the most recent user time for the app
+ * will be used.
+ *
+ * This function has no effect if @app is not currently running.
+ */
+void
+shell_app_activate_window (ShellApp     *app,
+                           MetaWindow   *window,
+                           guint32       timestamp)
+{
+  GSList *windows;
+
+  if (shell_app_get_state (app) != SHELL_APP_STATE_RUNNING)
+    return;
+
+  windows = shell_app_get_windows (app);
+  if (window == NULL && windows)
+    window = windows->data;
+
+  if (!g_slist_find (windows, window))
+    return;
+  else
+    {
+      GSList *iter;
+      ShellGlobal *global = shell_global_get ();
+      MetaScreen *screen = shell_global_get_screen (global);
+      MetaDisplay *display = meta_screen_get_display (screen);
+      MetaWorkspace *active = meta_screen_get_active_workspace (screen);
+      MetaWorkspace *workspace = meta_window_get_workspace (window);
+      guint32 last_user_timestamp = meta_display_get_last_user_time (display);
+      MetaWindow *most_recent_transient;
+
+      if (meta_display_xserver_time_is_before (display, timestamp, last_user_timestamp))
+        {
+          meta_window_set_demands_attention (window);
+          return;
+        }
+
+      /* Now raise all the other windows for the app that are on
+       * the same workspace, in reverse order to preserve the stacking.
+       */
+      for (iter = windows; iter; iter = iter->next)
+        {
+          MetaWindow *other_window = iter->data;
+
+          if (other_window != window)
+            meta_window_raise (other_window);
+        }
+
+      /* If we have a transient that the user's interacted with more recently than
+       * the window, pick that.
+       */
+      most_recent_transient = find_most_recent_transient_on_same_workspace (display, window);
+      if (most_recent_transient
+          && meta_display_xserver_time_is_before (display,
+                                                  meta_window_get_user_time (window),
+                                                  meta_window_get_user_time (most_recent_transient)))
+        window = most_recent_transient;
+
+      if (active != workspace)
+        meta_workspace_activate_with_focus (workspace, window, timestamp);
+      else
+        meta_window_activate (window, timestamp);
+    }
+}
+
+/**
+ * shell_app_activate:
+ * @app: a #ShellApp
+ *
+ * Perform an appropriate default action for operating on this application,
+ * dependent on its current state.  For example, if the application is not
+ * currently running, launch it.  If it is running, activate the most
+ * recently used NORMAL window (or if that window has a transient, the most
+ * recently used transient for that window).
+ */
+void
+shell_app_activate (ShellApp  *app)
+{
+  switch (app->state)
+    {
+      case SHELL_APP_STATE_STOPPED:
+        /* TODO sensibly handle this error */
+        shell_app_info_launch (app->info, NULL);
+        break;
+      case SHELL_APP_STATE_STARTING:
+        break;
+      case SHELL_APP_STATE_RUNNING:
+        shell_app_activate_window (app, NULL, shell_global_get_current_time (shell_global_get ()));
+        break;
+    }
+}
+
+/**
+ * shell_app_open_new_window:
+ * @app: a #ShellApp
+ *
+ * Request that the application create a new window.
+ */
+void
+shell_app_open_new_window (ShellApp *app)
+{
+  /* Here we just always launch the application again, even if we know
+   * it was already running.  For most applications this
+   * should have the effect of creating a new window, whether that's
+   * a second process (in the case of Calculator) or IPC to existing
+   * instance (Firefox).  There are a few less-sensical cases such
+   * as say Pidgin.  Ideally, we have the application express to us
+   * that it supports an explicit new-window action.
+   */
+  shell_app_info_launch (app->info, NULL);
+}
+
+/**
+ * shell_app_get_state:
+ * @app: a #ShellApp
+ *
+ * Returns: State of the application
+ */
+ShellAppState
+shell_app_get_state (ShellApp *app)
+{
+  return app->state;
 }
 
 /**
@@ -341,10 +562,9 @@ shell_app_is_on_workspace (ShellApp *app,
  * @other: A #ShellApp
  *
  * Compare one #ShellApp instance to another, in the following way:
+ *   - Running applications sort before not-running applications.
  *   - If one of them has visible windows and the other does not, the one
  *     with visible windows is first.
- *   - If one has no windows at all (i.e. it's not running) and the other
- *     does, the one with windows is first.
  *   - Finally, the application which the user interacted with most recently
  *     compares earlier.
  */
@@ -353,7 +573,13 @@ shell_app_compare (ShellApp *app,
                    ShellApp *other)
 {
   gboolean vis_app, vis_other;
-  GSList *windows_app, *windows_other;
+
+  if (app->state != other->state)
+    {
+      if (app->state == SHELL_APP_STATE_RUNNING)
+        return -1;
+      return 1;
+    }
 
   vis_app = shell_app_has_visible_windows (app);
   vis_other = shell_app_has_visible_windows (other);
@@ -368,10 +594,7 @@ shell_app_compare (ShellApp *app,
   else if (!app->windows && other->windows)
     return 1;
 
-  windows_app = shell_app_get_windows (app);
-  windows_other = shell_app_get_windows (other);
-
-  return meta_window_get_user_time (windows_other->data) - meta_window_get_user_time (windows_app->data);
+  return other->last_user_time - app->last_user_time;
 }
 
 ShellApp *
@@ -399,6 +622,18 @@ _shell_app_new (ShellAppInfo    *info)
   return app;
 }
 
+static void
+shell_app_state_transition (ShellApp      *app,
+                            ShellAppState  state)
+{
+  if (app->state == state)
+    return;
+  g_return_if_fail (!(app->state == SHELL_APP_STATE_RUNNING &&
+                      state == SHELL_APP_STATE_STARTING));
+  app->state = state;
+  g_object_notify (G_OBJECT (app), "state");
+}
+
 static void
 shell_app_on_unmanaged (MetaWindow      *window,
                         ShellApp *app)
@@ -411,6 +646,8 @@ shell_app_on_user_time_changed (MetaWindow *window,
                                 GParamSpec *pspec,
                                 ShellApp   *app)
 {
+  app->last_user_time = meta_window_get_user_time (window);
+
   /* Ideally we don't want to emit windows-changed if the sort order
    * isn't actually changing. This check catches most of those.
    */
@@ -437,6 +674,8 @@ void
 _shell_app_add_window (ShellApp        *app,
                        MetaWindow      *window)
 {
+  guint32 user_time;
+
   if (g_slist_find (app->windows, window))
     return;
 
@@ -445,6 +684,13 @@ _shell_app_add_window (ShellApp        *app,
   g_signal_connect (window, "notify::user-time", G_CALLBACK(shell_app_on_user_time_changed), app);
   app->window_sort_stale = TRUE;
 
+  user_time = meta_window_get_user_time (window);
+  if (user_time > app->last_user_time)
+    app->last_user_time = user_time;
+
+  if (app->state != SHELL_APP_STATE_RUNNING)
+    shell_app_state_transition (app, SHELL_APP_STATE_RUNNING);
+
   g_signal_emit (app, shell_app_signals[WINDOWS_CHANGED], 0);
 
   if (app->workspace_switch_id == 0)
@@ -484,12 +730,58 @@ _shell_app_remove_window (ShellApp   *app,
   g_signal_emit (app, shell_app_signals[WINDOWS_CHANGED], 0);
 
   if (app->windows == NULL)
-    disconnect_workspace_switch (app);
+    {
+      disconnect_workspace_switch (app);
+
+      shell_app_state_transition (app, SHELL_APP_STATE_STOPPED);
+    }
+}
+
+void
+_shell_app_set_starting (ShellApp        *app,
+                         gboolean         starting)
+{
+  if (starting && app->state == SHELL_APP_STATE_STOPPED)
+    shell_app_state_transition (app, SHELL_APP_STATE_STARTING);
+  else if (!starting && app->state == SHELL_APP_STATE_STARTING)
+    shell_app_state_transition (app, SHELL_APP_STATE_RUNNING);
+}
+
+/**
+ * shell_app_request_quit:
+ * @app: A #ShellApp
+ *
+ * Initiate an asynchronous request to quit this application.
+ * The application may interact with the user, and the user
+ * might cancel the quit request from the application UI.
+ *
+ * This operation may not be supported for all applications.
+ *
+ * Returns: %TRUE if a quit request is supported for this application
+ */
+gboolean
+shell_app_request_quit (ShellApp   *app)
+{
+  GSList *iter;
+
+  /* TODO - check for an XSMP connection; we could probably use that */
+
+  for (iter = app->windows; iter; iter = iter->next)
+    {
+      MetaWindow *win = iter->data;
+
+      if (!shell_window_tracker_is_window_interesting (win))
+        continue;
+
+      meta_window_delete (win, shell_global_get_current_time (shell_global_get ()));
+    }
+  return TRUE;
 }
 
 static void
 shell_app_init (ShellApp *self)
 {
+  self->state = SHELL_APP_STATE_STOPPED;
 }
 
 static void
@@ -516,6 +808,7 @@ shell_app_class_init(ShellAppClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
+  gobject_class->get_property = shell_app_get_property;
   gobject_class->dispose = shell_app_dispose;
 
   shell_app_signals[WINDOWS_CHANGED] = g_signal_new ("windows-changed",
@@ -525,4 +818,19 @@ shell_app_class_init(ShellAppClass *klass)
                                      NULL, NULL,
                                      g_cclosure_marshal_VOID__VOID,
                                      G_TYPE_NONE, 0);
+
+  /**
+   * ShellApp:state:
+   *
+   * The high-level state of the application, effectively whether it's
+   * running or not, or transitioning between those states.
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_STATE,
+                                   g_param_spec_enum ("state",
+                                                      "State",
+                                                      "Application state",
+                                                      SHELL_TYPE_APP_STATE,
+                                                      SHELL_APP_STATE_STOPPED,
+                                                      G_PARAM_READABLE));
 }
