@@ -74,6 +74,8 @@ void shell_global_breakpoint (ShellGlobal *global);
 
 void shell_global_gc (ShellGlobal *global);
 
+void shell_global_maybe_gc (ShellGlobal *global);
+
 void shell_global_format_time_relative_pretty (ShellGlobal *global, guint delta, char **text, guint *update_time);
 
 ClutterActor *shell_global_create_root_pixmap_actor (ShellGlobal *global);
@@ -82,17 +84,39 @@ GSList       *shell_global_get_monitors        (ShellGlobal  *global);
 GdkRectangle *shell_global_get_primary_monitor (ShellGlobal  *global);
 GdkRectangle *shell_global_get_focus_monitor   (ShellGlobal  *global);
 
-GdkModifierType shell_global_get_modifier_keys (ShellGlobal *global);
+void shell_global_get_pointer (ShellGlobal         *global,
+                               int                 *x,
+                               int                 *y,
+                               ClutterModifierType *mods);
 
 ClutterModifierType shell_get_event_state (ClutterEvent *event);
 
 void shell_popup_menu (GtkMenu *menu, int button, guint32 time,
                        int menu_x, int menu_y);
 
+gboolean shell_write_string_to_stream (GOutputStream *stream,
+                                       const char    *str,
+                                       GError       **error);
+
 guint32 shell_global_get_current_time (ShellGlobal *global);
 
 GAppLaunchContext *shell_global_create_app_launch_context (ShellGlobal *global);
 
+gboolean shell_global_set_property_mutable (ShellGlobal *global,
+                                            const char  *object,
+                                            const char  *property,
+                                            gboolean     mutable);
+
+void shell_global_begin_work     (ShellGlobal         *global);
+void shell_global_end_work       (ShellGlobal         *global);
+
+typedef void (*ShellLeisureFunction) (gpointer data);
+
+void shell_global_run_at_leisure (ShellGlobal         *global,
+                                  ShellLeisureFunction func,
+                                  gpointer             user_data,
+                                  GDestroyNotify       notify);
+
 G_END_DECLS
 
 #endif /* __SHELL_GLOBAL_H__ */
