@@ -16,7 +16,7 @@
 #include "shell-window-tracker.h"
 #include "shell-app-system.h"
 #include "shell-app-private.h"
-#include "shell-texture-cache.h"
+#include "st/st-texture-cache.h"
 #include "shell-global.h"
 #include "shell-marshal.h"
 
@@ -84,6 +84,7 @@ enum {
 enum {
   APP_RUNNING_CHANGED,
   STARTUP_SEQUENCE_CHANGED,
+  TRACKED_WINDOWS_CHANGED,
 
   LAST_SIGNAL
 };
@@ -91,7 +92,8 @@ enum {
 static guint signals[LAST_SIGNAL] = { 0 };
 
 static void shell_window_tracker_finalize (GObject *object);
-
+static void set_focus_app (ShellWindowTracker  *tracker,
+                           ShellApp            *new_focus_app);
 static void on_focus_window_changed (MetaDisplay *display, GParamSpec *spec, ShellWindowTracker *tracker);
 
 static void track_window (ShellWindowTracker *monitor, MetaWindow *window);
@@ -148,6 +150,13 @@ shell_window_tracker_class_init (ShellWindowTrackerClass *klass)
                                    NULL, NULL,
                                    g_cclosure_marshal_VOID__BOXED,
                                    G_TYPE_NONE, 1, SHELL_TYPE_STARTUP_SEQUENCE);
+  signals[TRACKED_WINDOWS_CHANGED] = g_signal_new ("tracked-windows-changed",
+                                                   SHELL_TYPE_WINDOW_TRACKER,
+                                                   G_SIGNAL_RUN_LAST,
+                                                   0,
+                                                   NULL, NULL,
+                                                   g_cclosure_marshal_VOID__VOID,
+                                                   G_TYPE_NONE, 0);
 }
 
 /**
@@ -286,16 +295,16 @@ shell_window_tracker_is_window_interesting (MetaWindow *window)
 }
 
 /**
- * get_app_for_window_direct:
+ * get_app_from_window_wmclass:
  *
  * Looks only at the given window, and attempts to determine
- * an application based on WM_CLASS.  If that fails, then
- * a "transient" application is created.
+ * an application based on WM_CLASS.  If one can't be determined,
+ * return %NULL.
  *
- * Return value: (transfer full): A newly-referenced #ShellApp
+ * Return value: (transfer full): A newly-referenced #ShellApp, or %NULL
  */
 static ShellApp *
-get_app_for_window_direct (MetaWindow  *window)
+get_app_from_window_wmclass (MetaWindow  *window)
 {
   ShellApp *app;
   ShellAppSystem *appsys;
@@ -306,7 +315,7 @@ get_app_for_window_direct (MetaWindow  *window)
   wmclass = get_appid_from_window (window);
 
   if (!wmclass)
-    return shell_app_system_get_app_for_window (appsys, window);
+    return NULL;
 
   with_desktop = g_strjoin (NULL, wmclass, ".desktop", NULL);
   g_free (wmclass);
@@ -322,12 +331,57 @@ get_app_for_window_direct (MetaWindow  *window)
         app = shell_app_system_get_app (appsys, id);
     }
 
-  if (app == NULL)
-    app = shell_app_system_get_app_for_window (appsys, window);
-
   return app;
 }
 
+/**
+ * get_app_from_window_group:
+ * @monitor: a #ShellWindowTracker
+ * @window: a #MetaWindow
+ *
+ * Check other windows in the group for @window to see if we have
+ * an application for one of them.
+ *
+ * Return value: (transfer full): A newly-referenced #ShellApp, or %NULL
+ */
+static ShellApp*
+get_app_from_window_group (ShellWindowTracker  *monitor,
+                           MetaWindow          *window)
+{
+  ShellApp *result;
+  GSList *group_windows;
+  MetaGroup *group;
+  GSList *iter;
+
+  group = meta_window_get_group (window);
+  if (group == NULL)
+    return NULL;
+
+  group_windows = meta_group_list_windows (group);
+
+  result = NULL;
+  /* Try finding a window in the group of type NORMAL; if we
+   * succeed, use that as our source. */
+  for (iter = group_windows; iter; iter = iter->next)
+    {
+      MetaWindow *group_window = iter->data;
+
+      if (meta_window_get_window_type (group_window) != META_WINDOW_NORMAL)
+        continue;
+
+      result = g_hash_table_lookup (monitor->window_to_app, group_window);
+      if (result)
+        break;
+    }
+
+  g_slist_free (group_windows);
+
+  if (result)
+    g_object_ref (result);
+
+  return result;
+}
+
 /**
  * get_app_for_window:
  *
@@ -340,44 +394,58 @@ get_app_for_window (ShellWindowTracker    *monitor,
                     MetaWindow         *window)
 {
   ShellApp *result;
-  MetaWindow *source_window;
-  GSList *group_windows;
-  MetaGroup *group;
-  GSList *iter;
-
-  group = meta_window_get_group (window);
-  if (group == NULL)
-    group_windows = g_slist_prepend (NULL, window);
-  else
-    group_windows = meta_group_list_windows (group);
-
-  source_window = window;
+  const char *startup_id;
 
   result = NULL;
-  /* Try finding a window in the group of type NORMAL; if we
-   * succeed, use that as our source. */
-  for (iter = group_windows; iter; iter = iter->next)
+  /* First, we check whether we already know about this window,
+   * if so, just return that.
+   */
+  if (meta_window_get_window_type (window) == META_WINDOW_NORMAL)
     {
-      MetaWindow *group_window = iter->data;
-
-      if (meta_window_get_window_type (group_window) != META_WINDOW_NORMAL)
-        continue;
-
-       source_window = group_window;
-       result = g_hash_table_lookup (monitor->window_to_app, group_window);
-       if (result)
-         break;
+      result = g_hash_table_lookup (monitor->window_to_app, window);
+      if (result != NULL)
+        {
+          g_object_ref (result);
+          return result;
+        }
     }
 
-  g_slist_free (group_windows);
-
+  /* Check if the app's WM_CLASS specifies an app */
+  result = get_app_from_window_wmclass (window);
   if (result != NULL)
+    return result;
+
+  /* Now we check whether we have a match through startup-notification */
+  startup_id = meta_window_get_startup_id (window);
+  if (startup_id)
     {
-      g_object_ref (result);
-      return result;
+      GSList *iter, *sequences;
+
+      sequences = shell_window_tracker_get_startup_sequences (monitor);
+      for (iter = sequences; iter; iter = iter->next)
+        {
+          ShellStartupSequence *sequence = iter->data;
+          const char *id = shell_startup_sequence_get_id (sequence);
+          if (strcmp (id, startup_id) != 0)
+            continue;
+
+          result = shell_startup_sequence_get_app (sequence);
+          if (result)
+            break;
+        }
     }
 
-  return get_app_for_window_direct (source_window);
+  /* If we didn't get a startup-notification match, see if we matched
+   * any other windows in the group.
+   */
+  if (result == NULL)
+    result = get_app_from_window_group (monitor, window);
+
+  /* Our last resort - we create a fake app from the window */
+  if (result == NULL)
+    result = shell_app_system_get_app_for_window (shell_app_system_get_default (), window);
+
+  return result;
 }
 
 const char *
@@ -454,6 +522,8 @@ track_window (ShellWindowTracker *self,
                            app);
       g_signal_emit (self, signals[APP_RUNNING_CHANGED], 0, app);
     }
+
+  g_signal_emit (self, signals[TRACKED_WINDOWS_CHANGED], 0);
 }
 
 static void
@@ -489,6 +559,8 @@ disassociate_window (ShellWindowTracker   *self,
        g_signal_emit (self, signals[APP_RUNNING_CHANGED], 0, app);
     }
 
+  g_signal_emit (self, signals[TRACKED_WINDOWS_CHANGED], 0);
+
   g_object_unref (app);
 }
 
@@ -575,7 +647,33 @@ on_startup_sequence_changed (MetaScreen            *screen,
                              SnStartupSequence     *sequence,
                              ShellWindowTracker    *self)
 {
-  /* Just proxy the signal */
+  ShellApp *app;
+
+  app = shell_startup_sequence_get_app ((ShellStartupSequence*)sequence);
+  if (app)
+    {
+      gboolean starting = !sn_startup_sequence_get_completed (sequence);
+
+      /* The Shell design calls for on application launch, the app title
+       * appears at top, and no X window is focused.  So when we get
+       * a startup-notification for this app, transition it to STARTING
+       * if it's currently stopped, set it as our application focus,
+       * but focus the no_focus window.
+       */
+      if (starting && shell_app_get_state (app) == SHELL_APP_STATE_STOPPED)
+        {
+          MetaScreen *screen = shell_global_get_screen (shell_global_get ());
+          MetaDisplay *display = meta_screen_get_display (screen);
+          long tv_sec, tv_usec;
+
+          sn_startup_sequence_get_initiated_time (sequence, &tv_sec, &tv_usec);
+
+          _shell_app_set_starting (app, starting);
+          set_focus_app (self, app);
+          meta_display_focus_the_no_focus_window (display, screen, tv_sec);
+        }
+    }
+
   g_signal_emit (G_OBJECT (self), signals[STARTUP_SEQUENCE_CHANGED], 0, sequence);
 }
 
@@ -711,52 +809,41 @@ shell_window_tracker_get_running_apps (ShellWindowTracker *monitor,
   return ret;
 }
 
-static gboolean
-idle_handle_focus_change (gpointer data)
+static void
+set_focus_app (ShellWindowTracker  *tracker,
+               ShellApp            *new_focus_app)
 {
-  MetaScreen *screen;
-  MetaDisplay *display;
-  ShellWindowTracker *tracker = data;
-  MetaWindow *new_focus_win;
-  ShellApp *new_focus_app;
-
-  tracker->idle_focus_change_id = 0;
-
-  screen = shell_global_get_screen (shell_global_get ());
-  display = meta_screen_get_display (screen);
-
-  new_focus_win = meta_display_get_focus_window (display);
-  new_focus_app = new_focus_win ? g_hash_table_lookup (tracker->window_to_app, new_focus_win) : NULL;
-
   if (new_focus_app == tracker->focus_app)
-    return FALSE;
+    return;
 
   if (tracker->focus_app != NULL)
     g_object_unref (tracker->focus_app);
 
-  if (tracker->focus_app != NULL
-      && (!new_focus_win || !new_focus_app))
-    tracker->focus_app = NULL;
-  else
-    tracker->focus_app = g_object_ref (new_focus_app);
+  tracker->focus_app = new_focus_app;
+
+  if (tracker->focus_app != NULL)
+    g_object_ref (tracker->focus_app);
 
   g_object_notify (G_OBJECT (tracker), "focus-app");
-
-  return FALSE;
 }
 
 static void
-on_focus_window_changed (MetaDisplay     *display,
-                         GParamSpec      *spec,
-                         ShellWindowTracker *self)
+on_focus_window_changed (MetaDisplay        *display,
+                         GParamSpec         *spec,
+                         ShellWindowTracker *tracker)
 {
-  if (self->idle_focus_change_id != 0)
-    return;
+  MetaScreen *screen;
+  MetaWindow *new_focus_win;
+  ShellApp *new_focus_app;
 
-  self->idle_focus_change_id = meta_later_add (META_LATER_BEFORE_REDRAW, idle_handle_focus_change, self, NULL);
+  screen = shell_global_get_screen (shell_global_get ());
+
+  new_focus_win = meta_display_get_focus_window (display);
+  new_focus_app = new_focus_win ? g_hash_table_lookup (tracker->window_to_app, new_focus_win) : NULL;
+
+  set_focus_app (tracker, new_focus_app);
 }
 
-
 /**
  * shell_window_tracker_get_startup_sequences:
  * @self:
@@ -799,6 +886,32 @@ shell_startup_sequence_get_id (ShellStartupSequence *sequence)
   return sn_startup_sequence_get_id ((SnStartupSequence*)sequence);
 }
 
+/**
+ * shell_startup_sequence_get_app:
+ * @sequence: A #ShellStartupSequence
+ *
+ * Returns: (transfer full): The application being launched, or %NULL if unknown.
+ */
+ShellApp *
+shell_startup_sequence_get_app (ShellStartupSequence *sequence)
+{
+#ifdef HAVE_SN_STARTUP_SEQUENCE_GET_APPLICATION_ID
+  const char *appid;
+  ShellAppSystem *appsys;
+  ShellApp *app;
+
+  appid = sn_startup_sequence_get_application_id ((SnStartupSequence*)sequence);
+  if (!appid)
+    return NULL;
+
+  appsys = shell_app_system_get_default ();
+  app = shell_app_system_get_app_for_path (appsys, appid);
+  return app;
+#else
+  return NULL;
+#endif
+}
+
 const char *
 shell_startup_sequence_get_name (ShellStartupSequence *sequence)
 {
@@ -834,8 +947,8 @@ shell_startup_sequence_create_icon (ShellStartupSequence *sequence, guint size)
     }
 
   themed = g_themed_icon_new (icon_name);
-  texture = shell_texture_cache_load_gicon (shell_texture_cache_get_default (),
-                                            themed, size);
+  texture = st_texture_cache_load_gicon (st_texture_cache_get_default (),
+                                         themed, size);
   g_object_unref (G_OBJECT (themed));
   return texture;
 }
