@@ -57,11 +57,10 @@
 #include "st-box-layout-child.h"
 
 
-
 static void st_box_container_iface_init (ClutterContainerIface *iface);
 static void st_box_scrollable_interface_init (StScrollableInterface *iface);
 
-G_DEFINE_TYPE_WITH_CODE (StBoxLayout, st_box_layout, ST_TYPE_WIDGET,
+G_DEFINE_TYPE_WITH_CODE (StBoxLayout, st_box_layout, ST_TYPE_CONTAINER,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
                                                 st_box_container_iface_init)
                          G_IMPLEMENT_INTERFACE (ST_TYPE_SCROLLABLE,
@@ -82,8 +81,6 @@ enum {
 
 struct _StBoxLayoutPrivate
 {
-  GList        *children;
-
   guint         spacing;
 
   guint         is_vertical : 1;
@@ -93,6 +90,17 @@ struct _StBoxLayoutPrivate
   StAdjustment *vadjustment;
 };
 
+/*
+ * ClutterContainer Interface Implementation
+ */
+static void
+st_box_sort_depth_order (ClutterContainer *container)
+{
+  /* The parent class' implementation would mess up the
+   * left-to-right order of the children - do nothing instead
+   */
+}
+
 /*
  * StScrollable Interface Implementation
  */
@@ -111,6 +119,8 @@ scrollable_set_adjustments (StScrollable *scrollable,
 {
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (scrollable)->priv;
 
+  g_object_freeze_notify (G_OBJECT (scrollable));
+
   if (hadjustment != priv->hadjustment)
     {
       if (priv->hadjustment)
@@ -130,6 +140,7 @@ scrollable_set_adjustments (StScrollable *scrollable,
         }
 
       priv->hadjustment = hadjustment;
+      g_object_notify (G_OBJECT (scrollable), "hadjustment");
     }
 
   if (vadjustment != priv->vadjustment)
@@ -151,7 +162,10 @@ scrollable_set_adjustments (StScrollable *scrollable,
         }
 
       priv->vadjustment = vadjustment;
+      g_object_notify (G_OBJECT (scrollable), "vadjustment");
     }
+
+  g_object_thaw_notify (G_OBJECT (scrollable));
 }
 
 static void
@@ -160,82 +174,14 @@ scrollable_get_adjustments (StScrollable  *scrollable,
                             StAdjustment **vadjustment)
 {
   StBoxLayoutPrivate *priv;
-  ClutterActor *actor, *stage;
 
   priv = (ST_BOX_LAYOUT (scrollable))->priv;
 
-  actor = CLUTTER_ACTOR (scrollable);
-  stage = clutter_actor_get_stage (actor);
-
-  if (hadjustment)
-    {
-      if (priv->hadjustment)
-        *hadjustment = priv->hadjustment;
-      else
-        {
-          StAdjustment *adjustment;
-          gdouble width, stage_width, increment;
-
-          if (stage)
-            {
-              width = clutter_actor_get_width (actor);
-              stage_width = clutter_actor_get_width (stage);
-              increment = MAX (1.0, MIN (stage_width, width));
-            }
-          else
-            {
-              width = increment = 1.0;
-            }
-
-          adjustment = st_adjustment_new (0,
-                                          0,
-                                          width,
-                                          1.0,
-                                          increment,
-                                          increment);
-
-          scrollable_set_adjustments (scrollable,
-                                      adjustment,
-                                      priv->vadjustment);
-
-          *hadjustment = adjustment;
-        }
-    }
+  if (priv->hadjustment)
+    *hadjustment = priv->hadjustment;
 
   if (vadjustment)
-    {
-      if (priv->vadjustment)
-        *vadjustment = priv->vadjustment;
-      else
-        {
-          StAdjustment *adjustment;
-          gdouble height, stage_height, increment;
-
-          if (stage)
-            {
-              height = clutter_actor_get_height (actor);
-              stage_height = clutter_actor_get_height (stage);
-              increment = MAX (1.0, MIN (stage_height, height));
-            }
-          else
-            {
-              height = increment = 1.0;
-            }
-
-          adjustment = st_adjustment_new (0,
-                                          0,
-                                          height,
-                                          1.0,
-                                          increment,
-                                          increment);
-
-          scrollable_set_adjustments (scrollable,
-                                      priv->hadjustment,
-                                      adjustment);
-
-          *vadjustment = adjustment;
-        }
-    }
+    *vadjustment = priv->vadjustment;
 }
 
 
@@ -247,172 +193,10 @@ st_box_scrollable_interface_init (StScrollableInterface *iface)
   iface->get_adjustments = scrollable_get_adjustments;
 }
 
-/*
- * ClutterContainer Implementation
- */
-static void
-st_box_container_add_actor (ClutterContainer *container,
-                            ClutterActor     *actor)
-{
-  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
-
-  clutter_actor_set_parent (actor, CLUTTER_ACTOR (container));
-
-  priv->children = g_list_append (priv->children, actor);
-
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
-
-  g_signal_emit_by_name (container, "actor-added", actor);
-}
-
-static void
-st_box_container_remove_actor (ClutterContainer *container,
-                               ClutterActor     *actor)
-{
-  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
-
-  GList *item = NULL;
-
-  item = g_list_find (priv->children, actor);
-
-  if (item == NULL)
-    {
-      g_warning ("Actor of type '%s' is not a child of container of type '%s'",
-                 g_type_name (G_OBJECT_TYPE (actor)),
-                 g_type_name (G_OBJECT_TYPE (container)));
-      return;
-    }
-
-  g_object_ref (actor);
-
-  priv->children = g_list_delete_link (priv->children, item);
-  clutter_actor_unparent (actor);
-
-  g_signal_emit_by_name (container, "actor-removed", actor);
-
-  g_object_unref (actor);
-
-  clutter_actor_queue_relayout ((ClutterActor*) container);
-}
-
-static void
-st_box_container_foreach (ClutterContainer *container,
-                          ClutterCallback   callback,
-                          gpointer          callback_data)
-{
-  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
-
-  g_list_foreach (priv->children, (GFunc) callback, callback_data);
-}
-
-static void
-st_box_container_lower (ClutterContainer *container,
-                        ClutterActor     *actor,
-                        ClutterActor     *sibling)
-{
-  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
-
-  /* copied from clutter/clutter/clutter-group.c */
-
-  priv->children = g_list_remove (priv->children, actor);
-
-  /* Push to bottom */
-  if (!sibling)
-    {
-      GList *last_item;
-
-      last_item = g_list_first (priv->children);
-
-      if (last_item)
-	sibling = last_item->data;
-
-      priv->children = g_list_prepend (priv->children, actor);
-    }
-  else
-    {
-      gint pos;
-
-      pos = g_list_index (priv->children, sibling);
-
-      priv->children = g_list_insert (priv->children, actor, pos);
-    }
-
-  /* See comment in group_raise for this */
-  if (sibling &&
-      clutter_actor_get_depth (sibling) != clutter_actor_get_depth (actor))
-    {
-      clutter_actor_set_depth (actor, clutter_actor_get_depth (sibling));
-    }
-
-  if (CLUTTER_ACTOR_IS_VISIBLE (container))
-    clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
-}
-
-static void
-st_box_container_raise (ClutterContainer *container,
-                        ClutterActor     *actor,
-                        ClutterActor     *sibling)
-{
-  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (container)->priv;
-
-  priv->children = g_list_remove (priv->children, actor);
-
-  /* copied from clutter/clutter/clutter-group.c */
-
-  /* Raise at the top */
-  if (!sibling)
-    {
-      GList *last_item;
-
-      last_item = g_list_last (priv->children);
-
-      if (last_item)
-	sibling = last_item->data;
-
-      priv->children = g_list_append (priv->children, actor);
-    }
-  else
-    {
-      gint pos;
-
-      pos = g_list_index (priv->children, sibling) + 1;
-
-      priv->children = g_list_insert (priv->children, actor, pos);
-    }
-
-  /* set Z ordering a value below, this will then call sort
-   * as values are equal ordering shouldn't change but Z
-   * values will be correct.
-   *
-   * FIXME: optimise
-   */
-  if (sibling &&
-      clutter_actor_get_depth (sibling) != clutter_actor_get_depth (actor))
-    {
-      clutter_actor_set_depth (actor, clutter_actor_get_depth (sibling));
-    }
-
-  if (CLUTTER_ACTOR_IS_VISIBLE (container))
-    clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
-}
-
-static void
-st_box_container_sort_depth_order (ClutterContainer *container)
-{
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
-}
-
 static void
 st_box_container_iface_init (ClutterContainerIface *iface)
 {
-  iface->add = st_box_container_add_actor;
-  iface->remove = st_box_container_remove_actor;
-  iface->foreach = st_box_container_foreach;
-  iface->lower = st_box_container_lower;
-  iface->raise = st_box_container_raise;
-  iface->sort_depth_order = st_box_container_sort_depth_order;
-
+  iface->sort_depth_order = st_box_sort_depth_order;
   iface->child_meta_type = ST_TYPE_BOX_LAYOUT_CHILD;
 }
 
@@ -491,9 +275,6 @@ st_box_layout_dispose (GObject *object)
 {
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (object)->priv;
 
-  while (priv->children)
-    clutter_actor_destroy (priv->children->data);
-
   if (priv->hadjustment)
     {
       g_object_unref (priv->hadjustment);
@@ -519,12 +300,14 @@ get_content_preferred_width (StBoxLayout *self,
   gint n_children = 0;
   gint n_fixed = 0;
   gfloat min_width, natural_width;
-  GList *l;
+  GList *l, *children;
 
   min_width = 0;
   natural_width = 0;
 
-  for (l = priv->children; l; l = g_list_next (l))
+  children = st_container_get_children_list (ST_CONTAINER (self));
+
+  for (l = children; l; l = g_list_next (l))
     {
       ClutterActor *child = l->data;
       gfloat child_min = 0, child_nat = 0;
@@ -550,7 +333,7 @@ get_content_preferred_width (StBoxLayout *self,
         }
       else
         {
-          clutter_container_child_get ((ClutterContainer*) self, child,
+          clutter_container_child_get (CLUTTER_CONTAINER (self), child,
                                        "y-fill", &child_fill,
                                        NULL);
           _st_actor_get_preferred_width (child, for_height, child_fill,
@@ -600,12 +383,14 @@ get_content_preferred_height (StBoxLayout *self,
   gint n_children = 0;
   gint n_fixed = 0;
   gfloat min_height, natural_height;
-  GList *l;
+  GList *l, *children;
 
   min_height = 0;
   natural_height = 0;
 
-  for (l = priv->children; l; l = g_list_next (l))
+  children = st_container_get_children_list (ST_CONTAINER (self));
+
+  for (l = children; l; l = g_list_next (l))
     {
       ClutterActor *child = l->data;
       gfloat child_min = 0, child_nat = 0;
@@ -665,11 +450,25 @@ st_box_layout_get_preferred_height (ClutterActor *actor,
                                     gfloat       *min_height_p,
                                     gfloat       *natural_height_p)
 {
+  StBoxLayout *self = ST_BOX_LAYOUT (actor);
+  StBoxLayoutPrivate *priv = self->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
 
   st_theme_node_adjust_for_width (theme_node, &for_width);
 
-  get_content_preferred_height (ST_BOX_LAYOUT (actor), for_width,
+  if (priv->hadjustment)
+    {
+      /* If we're scrolled, the parent calls us with the width that
+       * we'll actually get, which can be smaller than the minimum
+       * width that we give our contents.
+       */
+      gfloat min_width;
+
+      get_content_preferred_width (self, -1, &min_width, NULL);
+      for_width = MAX (for_width, min_width);
+    }
+
+  get_content_preferred_height (self, for_width,
                                 min_height_p, natural_height_p);
 
   st_theme_node_adjust_preferred_height (theme_node,
@@ -704,9 +503,11 @@ compute_shrinks (StBoxLayout *self,
                  gfloat       total_shrink)
 {
   StBoxLayoutPrivate *priv = self->priv;
-  int n_children = g_list_length (priv->children);
+  GList *children = st_container_get_children_list (ST_CONTAINER (self));
+  int n_children = g_list_length (children);
   BoxChildShrink *shrinks = g_new0 (BoxChildShrink, n_children);
-  gfloat shrink_so_far, base_shrink;
+  gfloat shrink_so_far;
+  gfloat base_shrink = 0; /* the "= 0" is just to make gcc happy */
   int n_shrink_children;
   GList *l;
   int i;
@@ -720,7 +521,7 @@ compute_shrinks (StBoxLayout *self,
 
   /* Find the amount of possible shrink for each child */
   int n_possible_shrink_children = 0;
-  for (l = priv->children, i = 0; l; l = l->next, i++)
+  for (l = children, i = 0; l; l = l->next, i++)
     {
       ClutterActor *child = l->data;
       gfloat child_min, child_nat;
@@ -821,7 +622,7 @@ st_box_layout_allocate (ClutterActor          *actor,
   ClutterActorBox content_box;
   gfloat avail_width, avail_height, min_width, natural_width, min_height, natural_height;
   gfloat position, next_position;
-  GList *l;
+  GList *l, *children;
   gint n_expand_children = 0, i;
   gfloat expand_amount, shrink_amount;
   BoxChildShrink *shrinks = NULL;
@@ -831,7 +632,8 @@ st_box_layout_allocate (ClutterActor          *actor,
   CLUTTER_ACTOR_CLASS (st_box_layout_parent_class)->allocate (actor, box,
                                                               flags);
 
-  if (priv->children == NULL)
+  children = st_container_get_children_list (ST_CONTAINER (actor));
+  if (children == NULL)
     return;
 
   st_theme_node_get_content_box (theme_node, box, &content_box);
@@ -839,10 +641,11 @@ st_box_layout_allocate (ClutterActor          *actor,
   avail_width  = content_box.x2 - content_box.x1;
   avail_height = content_box.y2 - content_box.y1;
 
-  get_content_preferred_height (ST_BOX_LAYOUT (actor), avail_width,
-                                &min_height, &natural_height);
   get_content_preferred_width (ST_BOX_LAYOUT (actor), avail_height,
                                &min_width, &natural_width);
+  get_content_preferred_height (ST_BOX_LAYOUT (actor), MAX (avail_width, min_width),
+                                &min_height, &natural_height);
+
 
   /* update adjustments for scrolling */
   if (priv->vadjustment)
@@ -851,10 +654,10 @@ st_box_layout_allocate (ClutterActor          *actor,
 
       g_object_set (G_OBJECT (priv->vadjustment),
                     "lower", 0.0,
-                    "upper", natural_height,
+                    "upper", MAX (min_height, avail_height),
                     "page-size", avail_height,
                     "step-increment", avail_height / 6,
-                    "page-increment", avail_height,
+                    "page-increment", avail_height - avail_height / 6,
                     NULL);
 
       prev_value = st_adjustment_get_value (priv->vadjustment);
@@ -867,32 +670,44 @@ st_box_layout_allocate (ClutterActor          *actor,
 
       g_object_set (G_OBJECT (priv->hadjustment),
                     "lower", 0.0,
-                    "upper", natural_width,
+                    "upper", MAX (min_width, avail_width),
                     "page-size", avail_width,
                     "step-increment", avail_width / 6,
-                    "page-increment", avail_width,
+                    "page-increment", avail_width - avail_width / 6,
                     NULL);
 
       prev_value = st_adjustment_get_value (priv->hadjustment);
       st_adjustment_set_value (priv->hadjustment, prev_value);
     }
 
+  if (avail_height < min_height)
+    {
+      avail_height = min_height;
+      content_box.y2 = content_box.y1 + avail_height;
+    }
+
+  if (avail_width < min_width)
+    {
+      avail_width = min_width;
+      content_box.x2 = content_box.x1 + avail_width;
+    }
+
   if (priv->is_vertical)
     {
       expand_amount = MAX (0, avail_height - natural_height);
-      shrink_amount = MAX (0, natural_height - MAX (avail_height, min_height));
+      shrink_amount = MAX (0, natural_height - avail_height);
     }
   else
     {
       expand_amount = MAX (0, avail_width - natural_width);
-      shrink_amount = MAX (0, natural_width - MAX (avail_width, min_width));
+      shrink_amount = MAX (0, natural_width - avail_width);
     }
 
   if (expand_amount > 0)
     {
       /* count the number of children with expand set to TRUE */
       n_expand_children = 0;
-      for (l = priv->children; l; l = l->next)
+      for (l = children; l; l = l->next)
         {
           ClutterActor *child = l->data;
           gboolean expand;
@@ -928,12 +743,12 @@ st_box_layout_allocate (ClutterActor          *actor,
 
   if (priv->is_pack_start)
     {
-      l = g_list_last (priv->children);
-      i = g_list_length (priv->children);
+      l = g_list_last (children);
+      i = g_list_length (children);
     }
   else
     {
-      l = priv->children;
+      l = children;
       i = 0;
     }
 
@@ -990,10 +805,7 @@ st_box_layout_allocate (ClutterActor          *actor,
           child_box.y1 = (int)(0.5 + position);
           child_box.y2 = (int)(0.5 + next_position);
           child_box.x1 = content_box.x1;
-          if (priv->hadjustment)
-            child_box.x2 = content_box.x1 + MAX (avail_width, natural_width);
-          else
-            child_box.x2 = content_box.x2;
+          child_box.x2 = content_box.x2;
 
           _st_allocate_fill (ST_WIDGET (actor), child, &child_box,
                              xalign, yalign, xfill, yfill);
@@ -1014,10 +826,7 @@ st_box_layout_allocate (ClutterActor          *actor,
             }
 
           child_box.y1 = content_box.y1;
-          if (priv->vadjustment)
-            child_box.y2 = content_box.y1 + MAX (avail_height, natural_height);
-          else
-            child_box.y2 = content_box.y2;
+          child_box.y2 = content_box.y2;
 
           _st_allocate_fill (ST_WIDGET (actor), child, &child_box,
                              xalign, yalign, xfill, yfill);
@@ -1074,9 +883,8 @@ st_box_layout_paint (ClutterActor *actor)
 {
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (actor)->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
-  GList *l;
+  GList *l, *children;
   gdouble x, y;
-  ClutterActorBox child_box;
   ClutterActorBox allocation_box;
   ClutterActorBox content_box;
 
@@ -1105,7 +913,9 @@ st_box_layout_paint (ClutterActor *actor)
       cogl_pop_matrix ();
     }
 
-  if (priv->children == NULL)
+  children = st_container_get_children_list (ST_CONTAINER (actor));
+
+  if (children == NULL)
     return;
 
   clutter_actor_get_allocation_box (actor, &allocation_box);
@@ -1120,27 +930,17 @@ st_box_layout_paint (ClutterActor *actor)
    * the borders and background stay in place; after drawing the borders and
    * background, we clip to the content area */
   if (priv->hadjustment || priv->vadjustment)
-    cogl_clip_push ((int)content_box.x1,
-                    (int)content_box.y1,
-                    (int)content_box.x2 - (int)content_box.x1,
-                    (int)content_box.y2 - (int)content_box.y1);
+    cogl_clip_push_rectangle ((int)content_box.x1,
+                              (int)content_box.y1,
+                              (int)content_box.x2,
+                              (int)content_box.y2);
 
-  for (l = priv->children; l; l = g_list_next (l))
+  for (l = children; l; l = g_list_next (l))
     {
       ClutterActor *child = (ClutterActor*) l->data;
 
-      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
-        continue;
-
-      clutter_actor_get_allocation_box (child, &child_box);
-
-      if ((child_box.x1 < content_box.x2) &&
-          (child_box.x2 > content_box.x1) &&
-          (child_box.y1 < content_box.y2) &&
-          (child_box.y2 > content_box.y1))
-        {
-          clutter_actor_paint (child);
-        }
+      if (CLUTTER_ACTOR_IS_VISIBLE (child))
+        clutter_actor_paint (child);
     }
 
   if (priv->hadjustment || priv->vadjustment)
@@ -1153,9 +953,8 @@ st_box_layout_pick (ClutterActor       *actor,
 {
   StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (actor)->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
-  GList *l;
+  GList *l, *children;
   gdouble x, y;
-  ClutterActorBox child_box;
   ClutterActorBox allocation_box;
   ClutterActorBox content_box;
 
@@ -1182,7 +981,9 @@ st_box_layout_pick (ClutterActor       *actor,
       cogl_pop_matrix ();
     }
 
-  if (priv->children == NULL)
+  children = st_container_get_children_list (ST_CONTAINER (actor));
+
+  if (children == NULL)
     return;
 
   clutter_actor_get_allocation_box (actor, &allocation_box);
@@ -1194,27 +995,17 @@ st_box_layout_pick (ClutterActor       *actor,
   content_box.y2 += y;
 
   if (priv->hadjustment || priv->vadjustment)
-    cogl_clip_push ((int)content_box.x1,
-                    (int)content_box.y1,
-                    (int)content_box.x2 - (int)content_box.x1,
-                    (int)content_box.y2 - (int)content_box.y1);
+    cogl_clip_push_rectangle ((int)content_box.x1,
+                              (int)content_box.y1,
+                              (int)content_box.x2,
+                              (int)content_box.y2);
 
-  for (l = priv->children; l; l = g_list_next (l))
+  for (l = children; l; l = g_list_next (l))
     {
       ClutterActor *child = (ClutterActor*) l->data;
 
-      if (!CLUTTER_ACTOR_IS_VISIBLE (child))
-        continue;
-
-      clutter_actor_get_allocation_box (child, &child_box);
-
-      if ((child_box.x1 < content_box.x2) &&
-          (child_box.x2 > content_box.x1) &&
-          (child_box.y1 < content_box.y2) &&
-          (child_box.y2 > content_box.y1))
-        {
-          clutter_actor_paint (child);
-        }
+      if (CLUTTER_ACTOR_IS_VISIBLE (child))
+        clutter_actor_paint (child);
     }
 
   if (priv->hadjustment || priv->vadjustment)
@@ -1309,7 +1100,7 @@ st_box_layout_new (void)
 /**
  * st_box_layout_set_vertical:
  * @box: A #StBoxLayout
- * @vertical: #TRUE if the layout should be vertical
+ * @vertical: %TRUE if the layout should be vertical
  *
  * Set the value of the #StBoxLayout::vertical property
  *
@@ -1335,7 +1126,7 @@ st_box_layout_set_vertical (StBoxLayout *box,
  *
  * Get the value of the #StBoxLayout::vertical property.
  *
- * Returns: #TRUE if the layout is vertical
+ * Returns: %TRUE if the layout is vertical
  */
 gboolean
 st_box_layout_get_vertical (StBoxLayout *box)
@@ -1348,7 +1139,7 @@ st_box_layout_get_vertical (StBoxLayout *box)
 /**
  * st_box_layout_set_pack_start:
  * @box: A #StBoxLayout
- * @pack_start: #TRUE if the layout should use pack-start
+ * @pack_start: %TRUE if the layout should use pack-start
  *
  * Set the value of the #StBoxLayout::pack-start property.
  *
@@ -1374,7 +1165,7 @@ st_box_layout_set_pack_start (StBoxLayout *box,
  *
  * Get the value of the #StBoxLayout::pack-start property.
  *
- * Returns: #TRUE if pack-start is enabled
+ * Returns: %TRUE if pack-start is enabled
  */
 gboolean
 st_box_layout_get_pack_start (StBoxLayout *box)
@@ -1384,94 +1175,11 @@ st_box_layout_get_pack_start (StBoxLayout *box)
   return box->priv->is_pack_start;
 }
 
-static void
-st_box_layout_internal_remove_all (StBoxLayout *self,
-                                   gboolean     destroy)
-{
-  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (self)->priv;
-  ClutterActor *child;
-
-  while (priv->children)
-    {
-      child = priv->children->data;
-
-      g_object_ref (child);
-      priv->children = g_list_delete_link (priv->children, priv->children);
-      clutter_actor_unparent (child);
-      g_signal_emit_by_name (self, "actor-removed", child);
-      if (destroy)
-        clutter_actor_destroy (child);
-      g_object_unref (child);
-    }
-
-  clutter_actor_queue_relayout ((ClutterActor*) self);
-}
-
-/**
- * st_box_layout_remove_all:
- * @self:
- *
- * Efficiently unparent all children currently in this box.
- */
-void
-st_box_layout_remove_all (StBoxLayout *self)
-{
-  st_box_layout_internal_remove_all (self, FALSE);
-}
-
-/**
- * st_box_layout_destroy_children:
- * @self:
- *
- * Efficiently unparent and destroy all children currently in this box.
- */
-void
-st_box_layout_destroy_children (StBoxLayout *self)
-{
-  st_box_layout_internal_remove_all (self, TRUE);
-}
-
-/**
- * st_box_layout_get_n_children:
- * @self: a #StBoxLayout
- *
- * Returns the number of children in this box.
- */
-guint
-st_box_layout_get_n_children  (StBoxLayout *self)
-{
-  return g_list_length (self->priv->children);
-}
-
-void
-st_box_layout_move_child (StBoxLayout  *self,
-                          ClutterActor *actor,
-                          int           pos)
-{
-  StBoxLayoutPrivate *priv = ST_BOX_LAYOUT (self)->priv;
-
-  GList *item = NULL;
-
-  item = g_list_find (priv->children, actor);
-
-  if (item == NULL)
-    {
-      g_warning ("Actor of type '%s' is not a child of the StBoxLayout container",
-                 g_type_name (G_OBJECT_TYPE (actor)));
-      return;
-    }
-
-  priv->children = g_list_delete_link (priv->children, item);
-  priv->children = g_list_insert (priv->children, actor, pos);
-  clutter_actor_queue_relayout ((ClutterActor*) self);
-}
-
 void
 st_box_layout_insert_actor (StBoxLayout  *self,
                             ClutterActor *actor,
                             int           pos)
 {
   clutter_container_add_actor((ClutterContainer*) self, actor);
-  st_box_layout_move_child(self, actor, pos);
+  st_container_move_child (ST_CONTAINER (self), actor, pos);
 }
-
